# Generated by roxygen2: do not edit by hand

S3method(print,repo)
export(repo_build)
export(repo_chunk)
export(repo_has)
export(repo_open)
import(digest)
import(tools)
importFrom(graphics,pie)
importFrom(utils,download.file)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
